module LessonlyApi
  class Groups < Request
    class << self
      # Retrieve all groups
      # https://docs.lessonly.com/#list-groups
      #
      # @return [ResourceType::Groups] the response converted to a resource object
      def list
        raw_result = send_get('/groups')
        LessonlyApi::ResourceType::Groups.new(raw_result)
      end

      # Retrieve a listing of a groups members and managers.
      # https://docs.lessonly.com/#show-group-details
      #
      # @param id [Integer] ID of the group
      # @return [ResourceType::Group] the response converted to a resource object
      def show(id)
        raw_result = send_get("/groups/#{id}")
        LessonlyApi::ResourceType::Group.new(raw_result)
      end

      # Create a group
      # https://docs.lessonly.com/#create-group
      #
      # @param attributes [Hash] hash with group's attributes
      # @return [ResourceType::Group] the response converted to a resource object
      def create(attributes = {})
        raw_result = send_post('/groups', attributes)
        ResourceType::Group.new(raw_result)
      end

      # Update a group
      # https://docs.lessonly.com/#update-group
      #
      # @param id [Integer] ID of a group to update
      # @param attributes [Hash] hash with group's attributes to update
      # @return [ResourceType::Group] the response converted to a resource object
      def update(id, attributes = {})
        raw_result = send_put("/groups/#{id}", attributes)
        ResourceType::Group.new(raw_result)
      end

      # Archive a group
      # https://docs.lessonly.com/#archive-group
      #
      # @param id [Integer] ID of a group to archive
      # @return [ResourceType::Group] the response converted to a resource object
      def archive(id)
        raw_result = send_put("/groups/#{id}/archive")
        ResourceType::Group.new(raw_result)
      end

      # Restore an archived group
      # https://docs.lessonly.com/#restore-group
      #
      # @param id [Integer] ID of a group to restore
      # @return [ResourceType::Group] the response converted to a resource object
      def restore(id)
        raw_result = send_put("/groups/#{id}/restore")
        ResourceType::Group.new(raw_result)
      end

      # Delete a group
      # https://docs.lessonly.com/#delete-group
      #
      # @param id [Integer] ID of a group to delete
      # @return [ResourceType::Group] the response converted to a resource object
      def delete(id)
        raw_result = send_delete("/groups/#{id}")
        ResourceType::Group.new(raw_result)
      end

      # Return a list of assignments for a given group
      # https://docs.lessonly.com/#group-assignments
      #
      # @param id [Integer] ID of a group
      # @return [ResourceType::GroupAssignments] the response converted to a resource object
      def group_assignments(id)
        raw_result = send_get("/groups/#{id}/assignments")
        raw_result.map { |el| ResourceType::GroupAssignment.new(el) }
      end
    end
  end
end
