# frozen_string_literal: true

require 'active_model'
require 'active_support/core_ext/hash'
require 'json'

module LessonlyApi
  module ResourceType
    class Base
      include ActiveModel::Model

      class_attribute :fields_names
      self.fields_names = []

      class << self
        def from_json(json)
          parsed_json = json && JSON.parse(json).to_h
          return new({}) unless parsed_json

          new(parsed_json)
        end

        def fields(*keys_with_types)
          keys_with_types.each do |name_or_hash|
            field_name = define_methods_for(name_or_hash)
            self.fields_names += [field_name]
          end
        end

        def load(string)
          from_json(string)
        end

        def dump(data_type)
          data_type.to_json
        end

        private

        def define_methods_for(name_or_hash)
          if name_or_hash.is_a?(Hash)
            define_custom_field(*name_or_hash.to_a.first)
          else
            define_simple_field(name_or_hash)
          end
        end

        def define_custom_field(name, data_type)
          attr_reader name

          define_method("#{name}=") do |value|
            value = LessonlyApi::Utils::CustomFieldConverter.new(data_type).convert(value)
            instance_variable_set("@#{name}", value)
          end

          name
        end

        def define_simple_field(name)
          attr_accessor name
          name
        end
      end

      def initialize(attributes = {})
        defined_attributes = attributes.with_indifferent_access.slice(*fields_names)

        super(defined_attributes)
      end

      def attributes
        fields_names.each_with_object({}) do |field, hash|
          value = public_send(field)
          hash[field] = value if value
        end
      end
    end
  end
end
