require 'delegate'
require 'faraday'
require 'base64'
require 'json'

module LessonlyApi
  module Utils
    class Client
      extend Forwardable

      ERRORS_MAPPING = {
        400 => LessonlyApi::Errors::BadRequest,
        401 => LessonlyApi::Errors::Unauthorized,
        403 => LessonlyApi::Errors::Forbidden,
        404 => LessonlyApi::Errors::NotFound,
        405 => LessonlyApi::Errors::MethodNotAllowed,
        406 => LessonlyApi::Errors::NotAcceptable,
        500 => LessonlyApi::Errors::InternalServerError,
        503 => LessonlyApi::Errors::ServiceUnavailable
      }.freeze

      def_delegators :@logger, :debug, :info, :warn, :error, :fatal

      attr_reader :root_url, :domain, :api_key, :logger

      def initialize(configuration)
        @root_url = configuration.root_url
        @domain = configuration.domain
        @api_key = configuration.api_key
        @logger = configuration.logger
      end

      def self.instance
        @instance ||= new(LessonlyApi.configuration)
      end

      def send_get(endpoint, params = {})
        send_request(:get, endpoint, params)
      end

      def send_post(endpoint, params = {})
        send_request(:post, endpoint, params)
      end

      def send_put(endpoint, params = {})
        send_request(:put, endpoint, params)
      end

      def send_delete(endpoint, params = {})
        send_request(:delete, endpoint, params)
      end

      private

      def connection
        Faraday.new(root_url, http_options) do |faraday|
          faraday.request :url_encoded
          faraday.response :logger, logger, headers: true, bodies: true

          faraday.adapter :httpclient
        end
      end

      def http_options
        {
          headers: {
            'Accept' => 'application/json',
            'Content-Type' => 'application/json',
            'Authorization' => auth_token
          }
        }
      end

      def auth_token
        Base64.strict_encode64("#{domain}:#{api_key}")
      end

      def send_request(method, endpoint, params)
        response = with_tag do
          connection.public_send(method) do |request|
            # TODO: i.kamenko refactor it
            request.url "/api/v1.1/#{endpoint}"
            assign_params(request, method: method, params: params)
          end
        end

        process_response(response)
      rescue Faraday::TimeoutError => e
        raise LessonlyApi::Errors::ApiTimeoutError, e.message
      rescue Faraday::Error => e
        raise LessonlyApi::Errors::Base, e.message
      end

      def assign_params(request, method:, params:)
        if method == :get
          request.params = params
        else
          request.body = params.to_json
        end
      end

      def process_response(response)
        return process_success(response) if response.status >= 200 && response.status < 300

        error = ERRORS_MAPPING[response.status] || LessonlyApi::Errors::ApiError
        raise(error, parse_json(response.body))
      end

      def process_success(response)
        return {} if response.body.empty?

        parse_json(response.body)
      end

      def with_tag
        if logger&.respond_to?(:tagged)
          logger.tagged('LESSONLY API') { yield }
        else
          yield
        end
      end

      def parse_json(data)
        JSON.parse(data)
      rescue JSON::ParserError => e
        raise LessonlyApi::Errors::InvalidJson, e.message
      end
    end
  end
end
